package java_util;

import java.util.*;
import org.junit.*;
import static org.junit.Assert.*;

/**
 *
 * @author Ulf Zibis <Ulf.Zibis at CoSoCo.de>
 */
public class ScannerTest {

    static final String TEXT_AND_EMPTY_LINE = "Line0\nLine1\n\n";
    static final String INDENTED_TEXT_LINE = "Line0\n    Line1\n";
    static final String EMPTY_LINE = "Line0\n\n";
    static final String EMPTY_LINE_AND_TEXT = "Line0\n\nLine2";
    static final String WHITE_LINE_AND_TEXT = "Line0\n    \nLine2";

    static final String SKIP_EMPTY_LINE_PATTERN = "\\s*$";
    static final String SKIP_EMPTY_LINE_PATTERN_2 = "(?m)\\s*$";
    static final String ADJACENT_EMPTY_LINE_PATTERN = "^\\s*$";
    static final String ANY_EMPTY_LINE_PATTERN = "(?m)^\\s*$";
    Scanner scanner;
    String result;

    public ScannerTest() {
    }

    @Test
    public void skip() {
        scanner = new Scanner(TEXT_AND_EMPTY_LINE);
        scanner.nextLine();
        try { scanner.skip(SKIP_EMPTY_LINE_PATTERN); }
        catch(NoSuchElementException nsee) {}
        result = scanner.nextLine();
        assertEquals("Line1", result);         // fine because of erroneous pattern!
        scanner = new Scanner(INDENTED_TEXT_LINE);
        scanner.nextLine();
        try { scanner.skip(SKIP_EMPTY_LINE_PATTERN); }
        catch(NoSuchElementException nsee) {}
        result = scanner.nextLine();
        assertEquals("    Line1", result);         // fine because of erroneous pattern!
        scanner = new Scanner(EMPTY_LINE);
        scanner.nextLine();
        scanner.skip(SKIP_EMPTY_LINE_PATTERN);
        result = scanner.nextLine();         // error!
        assertEquals(null, result);
        scanner = new Scanner(EMPTY_LINE_AND_TEXT);
        scanner.nextLine();
        try { scanner.skip(SKIP_EMPTY_LINE_PATTERN); }
        catch(NoSuchElementException nsee) {}
        result = scanner.nextLine();
        assertEquals("Line2", result);         // error!
        scanner = new Scanner(WHITE_LINE_AND_TEXT);
        scanner.nextLine();
        try { scanner.skip(SKIP_EMPTY_LINE_PATTERN); }
        catch(NoSuchElementException nsee) {}
        result = scanner.nextLine();
        assertEquals("Line2", result);         // error!
    }

    @Test
    public void skip2() {
        scanner = new Scanner(TEXT_AND_EMPTY_LINE);
        scanner.nextLine();
        try { scanner.skip(SKIP_EMPTY_LINE_PATTERN_2); }
        catch(NoSuchElementException nsee) {
            result = "next line not empty"; }
        if (scanner.hasNextLine()) {     // if present,
            result = scanner.nextLine(); // ...read line n
            assertEquals("Line1", result);
        }

        scanner = new Scanner(INDENTED_TEXT_LINE);
        scanner.nextLine();
        try { scanner.skip(SKIP_EMPTY_LINE_PATTERN_2); }
        catch(NoSuchElementException nsee) {
            result = "next line not empty"; }
        if (scanner.hasNextLine()) {     // if present,
            result = scanner.nextLine(); // ...read line n
            assertEquals("    Line1", result);
        }

        scanner = new Scanner(EMPTY_LINE);
        scanner.nextLine();
        scanner.skip(SKIP_EMPTY_LINE_PATTERN_2);
        assertFalse(scanner.hasNextLine());

        scanner = new Scanner(EMPTY_LINE_AND_TEXT);
        scanner.nextLine();
        try { scanner.skip(SKIP_EMPTY_LINE_PATTERN_2); }
        catch(NoSuchElementException nsee) {
            result = "next line not empty"; } // here NetBeans IDE bug !!!
        if (scanner.hasNextLine()) {     // if present,
            result = scanner.nextLine(); // ...read line n+1
            assertEquals("Line2", result);         // >------->> error!
        }

        scanner = new Scanner(WHITE_LINE_AND_TEXT);
        scanner.nextLine();
        try { scanner.skip(SKIP_EMPTY_LINE_PATTERN_2); }
        catch(NoSuchElementException nsee) {
            result = "next line not empty"; } // here NetBeans IDE bug !!!
        if (scanner.hasNextLine()) {     // if present,
            result = scanner.nextLine(); // ...read line n+1
            assertEquals("Line2", result);         // >------->> error!
        }

    }

    @Test
    // Problem: Can't find any suitable pattern to skip possible empty line,
    // so tried with findInLine, but found errors :-(
    public void findInLine() {
        scanner = new Scanner(TEXT_AND_EMPTY_LINE);
        scanner.nextLine();
        result = scanner.findInLine(ADJACENT_EMPTY_LINE_PATTERN);
        assertEquals(null, result);
        if (result != null)
            scanner.nextLine();          // no line to skip
        if (scanner.hasNextLine()) {     // if present,
            result = scanner.nextLine(); // ...read line n
            assertEquals("Line1", result);
        }

        scanner = new Scanner(INDENTED_TEXT_LINE);
        scanner.nextLine();
        result = scanner.findInLine(ADJACENT_EMPTY_LINE_PATTERN);
        assertEquals(null, result);
        if (result != null)
            scanner.nextLine();          // no line to skip
        if (scanner.hasNextLine()) {     // if present,
            result = scanner.nextLine(); // ...read line n
            assertEquals("    Line1", result);
        }

        scanner = new Scanner(EMPTY_LINE);
        scanner.nextLine();
        result = scanner.findInLine(ADJACENT_EMPTY_LINE_PATTERN);
        assertEquals("", result);         // >------->> error!
        if (result != null)
            scanner.nextLine();          // skip line
        if (scanner.hasNextLine()) {     // if present,
            result = scanner.nextLine(); // ...read line n+1
            assertEquals(null, result);
        }

        scanner = new Scanner(EMPTY_LINE_AND_TEXT);
        scanner.nextLine();
        result = scanner.findInLine(ADJACENT_EMPTY_LINE_PATTERN);
        assertEquals("", result);         // >------->> error!
        if (result != null)
            scanner.nextLine();          // skip line
        if (scanner.hasNextLine()) {     // if present,
            result = scanner.nextLine(); // ...read line n+1
            assertEquals("Line2", result);
        }

        scanner = new Scanner(WHITE_LINE_AND_TEXT);
        scanner.nextLine();
        result = scanner.findInLine(ADJACENT_EMPTY_LINE_PATTERN);
        assertEquals("    ", result);         // >------->> error!
        if (result != null)
            scanner.nextLine();          // skip line
        if (scanner.hasNextLine()) {     // if present,
            result = scanner.nextLine(); // ...read line n+1
            assertEquals("Line2", result);
        }
    }

    @Test
    public void findWithinHorizon() {
        scanner = new Scanner(TEXT_AND_EMPTY_LINE);
        scanner.nextLine();
        result = scanner.findWithinHorizon(ANY_EMPTY_LINE_PATTERN, 1);
        assertEquals(null, result);
        scanner = new Scanner(INDENTED_TEXT_LINE);
        scanner.nextLine();
        result = scanner.findWithinHorizon(ANY_EMPTY_LINE_PATTERN, 1);
        assertEquals(null, result);
        scanner = new Scanner(EMPTY_LINE_AND_TEXT);
        scanner.nextLine();
        result = scanner.findWithinHorizon(ANY_EMPTY_LINE_PATTERN, 1);
        assertEquals("", result);         // fine!
        scanner = new Scanner(EMPTY_LINE_AND_TEXT);
        scanner.nextLine();
        result = scanner.findWithinHorizon(ANY_EMPTY_LINE_PATTERN, 0);
        assertEquals("", result);         // fine!
        scanner = new Scanner(WHITE_LINE_AND_TEXT);
        scanner.nextLine();
        result = scanner.findWithinHorizon(ANY_EMPTY_LINE_PATTERN, WHITE_LINE_AND_TEXT.length() - (WHITE_LINE_AND_TEXT.indexOf('\n') + 1));
        assertEquals("    ", result);         // fine!
        scanner = new Scanner(EMPTY_LINE);
        scanner.nextLine();
        result = scanner.findWithinHorizon(ANY_EMPTY_LINE_PATTERN, 1);
        assertEquals("", result);         // error!
        scanner = new Scanner(TEXT_AND_EMPTY_LINE);
        scanner.nextLine();
        result = scanner.findWithinHorizon(ANY_EMPTY_LINE_PATTERN, 0);
        assertEquals("", result);         // error!
        String whiteLineAtBegin = WHITE_LINE_AND_TEXT.substring(WHITE_LINE_AND_TEXT.indexOf('\n') + 1);
        scanner = new Scanner(whiteLineAtBegin);
        scanner.nextLine();
        result = scanner.findWithinHorizon(ANY_EMPTY_LINE_PATTERN, whiteLineAtBegin.length());
        assertEquals("    ", result);         // error!
    }
}